# -*- coding: utf-8 -*-
"""
Created on Tue Jul  2 09:04:41 2019

@author: michaelek
"""
import io
import numpy as np
from gistools import vector
from allotools import AlloUsage
# from hydrolm import LM
import os
import sys
import pandas as pd
import geopandas as gpd
import nzrec
import booklet

# try:
#     import plotly.offline as py
#     import plotly.graph_objs as go
# except:
#     print('install plotly for plot functions to work')

#####################################
### Parameters

# base_dir = os.path.dirname(os.path.abspath( __file__ ))
# base_dir = os.path.realpath(os.path.dirname(__file__))

# print(base_dir)

# with open(os.path.join(base_dir, 'parameters.yml')) as param2:
#     param = yaml.safe_load(param2)

# # datasets_path = os.path.join(base_dir, 'datasets')
# outputs = param['output']

# catch_key_base = 'tethys/station_misc/{station_id}.catchment.geojson.zst'

####################################
### Testing

# base_dir = os.path.split(os.path.realpath(os.path.dirname(__file__)))[0]

# with open(os.path.join(base_dir, 'parameters.yml')) as param2:
#     param1 = yaml.safe_load(param2)

# flow_remote = param1['remote']['flow']
# usage_remote = param1['remote']['usage']
#

# flow_stations_path = '/home/mike/git/HRC-flow-nat/data/flow_sites.gpkg'
# flow_data_path = '/home/mike/git/HRC-flow-nat/data/flow_data_daily.blt'
# permits_path = '/home/mike/git/HRC-flow-nat/data/permits.blt'
# usage_path = '/home/mike/git/HRC-flow-nat/data/abstraction_data_daily.blt'
# nzrec_path = '/home/mike/git/nzrec/data'


# from_date='2010-07-01'
# to_date='2020-06-30'
# product_code='quality_controlled_data'
# min_gaugings=10
# output_path=os.path.join(base_dir, 'tests')
# local_tz='Etc/GMT-12'
# station_id=['0bc0762fac7423261610b50f', '0ba603f66f55a19d18cbeb81', '0c6b76f9ff6fcf2e103f5e84', '2ec4a2cfa71dd4811eec25e4', '0d1024b9975b573e515ebd62']
# station_id=['0d1024b9975b573e515ebd62']
# ref=None
#
#
# self = FlowNat(flow_remote, usage_remote, from_date, to_date, product_code, min_gaugings, station_id, ref, output_path)
#
# stns_all = self.stations_all.station_id.unique().tolist().copy()
#
# stns1 = self.process_stations(stns_all)
#
# nat_flow = self.naturalisation()


# wap1 = 'SW/0082'
#
# a1 = AlloUsage(from_date='2015-06-30', to_date='2016-06-30', wap_filter={'wap': [wap1]})
#
# res1 = a1.get_ts(['allo', 'usage'], 'D', ['wap'])

#######################################
### Class


class FlowNat(object):
    """
    Class to perform several operations to ultimately naturalise flow data.
    Initialise the class with the following parameters.

    Parameters
    ----------
    from_date : str
        The start date for the flow record.
    to_date : str
        The end of of the flow record.
    min_gaugings : int
        The minimum number of gaugings required for the regressions. Default is 8.
    rec_data_code : str
        Either 'RAW' for the raw telemetered recorder data, or 'Primary' for the quality controlled recorder data. Default is 'Primary'.
    input_sites : str, int, list, or None
        Flow sites (either recorder or gauging) to be naturalised. If None, then the input_sites need to be defined later. Default is None.
    output_path : str or None
        Path to save the processed data, or None to not save them.
    load_rec : bool
        should the REC rivers and catchment GIS layers be loaded in at initiation?

    Returns
    -------
    FlowNat instance
    """
    def __init__(self, flow_stations_path, flow_data_path, permits_path, usage_path, nzrec_path, from_date=None, to_date=None, use_type_mapping={}, default_sd_ratio=0.35, output_path=None):
        """
        Class to perform several operations to ultimately naturalise flow data.
        Initialise the class with the following parameters.

        Parameters
        ----------
        from_date : str
            The start date for the flow record.
        to_date : str
            The end of of the flow record.
        min_gaugings : int
            The minimum number of gaugings required for the regressions. Default is 8.
        rec_data_code : str
            Either 'RAW' for the raw telemetered recorder data, or 'Primary' for the quality controlled recorder data. Default is 'Primary'.
        input_sites : str, int, list, or None
            Flow sites (either recorder or gauging) to be naturalised. If None, then the input_sites need to be defined later. Default is None.
        output_path : str or None
            Path to save the processed data, or None to not save them.
        catch_del : str
            Defines what should be used for the catchments associated with flow sites. 'rec' will perform a catchment delineation on-the-fly using the REC rivers and catchments GIS layers, 'internal' will use the pre-generated catchments stored in the package, or a path to a shapefile will use a user created catchments layer. The shapefile must at least have a column named ExtSiteID with the flow site numbers associated with the catchment geometry.

        Returns
        -------
        FlowNat instance
        """
        setattr(self, 'from_date', from_date)
        setattr(self, 'to_date', to_date)
        # setattr(self, 'min_gaugings', min_gaugings)
        # setattr(self, 'flow_remote', flow_remote)
        # setattr(self, 'usage_remote', usage_remote)
        # setattr(self, 'product_code', product_code)
        # setattr(self, 'local_tz', local_tz)

        self.flow_stations_path = flow_stations_path
        self.flow_data_path = flow_data_path
        self.permits_path = permits_path
        self.usage_path = usage_path
        self.nzrec_path = nzrec_path
        self.default_sd_ratio = default_sd_ratio
        self.use_type_mapping = use_type_mapping

        self.save_path(output_path)

        _ = self.get_all_flow_stations()

        allo0 = AlloUsage(permits_path, usage_path, from_date, to_date, use_type_mapping=use_type_mapping, default_sd_ratio=default_sd_ratio)

        self.waps_all = vector.xy_to_gpd(['permit_id', 'wap', 'sd_ratio'], 'lon', 'lat', allo0.waps, 4326)
        self.permits_all = allo0.permits

        # if (isinstance(station_id, list)) or (isinstance(ref, list)):
        #     _ = self.process_stations(station_id=station_id, ref=ref)


    def save_path(self, output_path=None):
        """

        """
        if output_path is None:
            pass
        elif isinstance(output_path, str):
            if not os.path.exists(output_path):
                os.makedirs(output_path)

            setattr(self, 'output_path', output_path)


    def get_all_flow_stations(self):
        """
        Function to process the flow datasets

        Parameters
        ----------
        from_date : str
            The start date for the flow record.
        to_date : str
            The end of of the flow record.
        min_gaugings : int
            The minimum number of gaugings required for the regressions. Default is 8.
        rec_data_code : str
            Either 'RAW' for the raw telemetered recorder data, or 'Primary' for the quality controlled recorder data. Default is 'Primary'.

        Returns
        -------
        DataFrame
        """
        # tethys1 = Tethys([self.flow_remote])

        # flow_ds = [ds for ds in tethys1.datasets if (ds['parameter'] == 'streamflow') and (ds['product_code'] == self.product_code) and (ds['frequency_interval'] == '24H') and (ds['utc_offset'] == '12H') and (ds['method'] == 'sensor_recording')]
        # flow_ds.extend([ds for ds in tethys1.datasets if (ds['parameter'] == 'streamflow') and (ds['product_code'] == self.product_code) and (ds['frequency_interval'] == 'None') and (ds['method'] == 'field_activity')])

        # stns_list = []

        # for ds in flow_ds:
        #     stns1 = tethys1.get_stations(ds['dataset_id'])
        #     stns_list.extend(stns1)

        # stns_list2 = [s for s in stns_list if s['dimensions']['time'] >= self.min_gaugings]

        # stns_list3 = [{'dataset_id': s['dataset_id'], 'station_id': s['station_id'], 'ref': s['ref'], 'geometry': Point(s['geometry']['coordinates']), 'count': s['dimensions']['time'], 'from_date': s['time_range']['from_date'], 'to_date': s['time_range']['to_date']} for s in stns_list2]
        # [s.update({'from_date': s['from_date'] + '+00:00', 'to_date': s['to_date'] + '+00:00'}) for s in stns_list3 if not '+00:00' in s['from_date']]

        # stns_summ = gpd.GeoDataFrame(pd.DataFrame(stns_list3), geometry='geometry', crs=4326)
        # try:
        #     stns_summ['from_date'] = pd.to_datetime(stns_summ['from_date']).dt.tz_convert(self.local_tz).dt.tz_localize(None)
        #     stns_summ['to_date'] = pd.to_datetime(stns_summ['to_date']).dt.tz_convert(self.local_tz).dt.tz_localize(None)
        # except:
        #     pass

        # if isinstance(self.from_date, str):
        #     from_date1 = pd.Timestamp(self.from_date)
        #     stns_summ = stns_summ[stns_summ['from_date'] <= from_date1]
        # if isinstance(self.to_date, str):
        #     to_date1 = pd.Timestamp(self.to_date)
        #     stns_summ = stns_summ[stns_summ['to_date'] >= to_date1]

        # setattr(self, 'stations_all', stns_summ)
        # setattr(self, '_tethys_flow', tethys1)
        # setattr(self, 'flow_datasets_all', flow_ds)

        self.stations_all = gpd.read_file(self.flow_stations_path)

        return self.stations_all


    def process_stations(self, station_ids=None, refs=None):
        """
        Function to process the sites.

        Parameters
        ----------
        input_sites : str, int, list, or None
            Flow sites (either recorder or gauging) to be naturalised. If None, then the input_sites need to be defined later. Default is None.

        Returns
        -------
        DataFrame
        """
        ## Checks
        # if (not isinstance(station_ids, list)) and (not isinstance(refs, list)):
        #     raise ValueError('station_ids and refs must be lists')

        ## Filter
        stns1 = self.stations_all.copy()

        bad_stns = []

        if isinstance(station_ids, list):
            stns1 = stns1[stns1['station_id'].isin(station_ids)]
            [bad_stns.extend([s['ref']]) for i, s in stns1.iterrows() if s['station_id'] not in station_ids]
        if isinstance(refs, list):
            stns1 = stns1[stns1['ref'].isin(refs)]
            [bad_stns.extend([s['ref']]) for i, s in stns1.iterrows() if s['ref'] not in refs]

        if bad_stns:
            print(', '.join(bad_stns) + ' stations are not available for naturalisation')

        ## Save if required
        # if hasattr(self, 'output_path'):
        #     run_time = pd.Timestamp.today().strftime('%Y-%m-%dT%H%M')
        #     flow_sites_shp = outputs['flow_sites_shp'].format(run_date=run_time)
        #     save1 = stns1.copy()
        #     save1['from_date'] = save1['from_date'].astype(str)
        #     save1['to_date'] = save1['to_date'].astype(str)
        #     save1.to_file(os.path.join(self.output_path, flow_sites_shp))

        ## Drop duplicate stations
        stns2 = stns1.drop_duplicates('station_id').rename(columns={'SiteName': 'ref'})

        setattr(self, 'stations', stns2)

        ## Filter flow datasets
        # stn_ds = stns2['dataset_id'].unique()
        # flow_ds1 = self.flow_datasets_all.copy()
        # flow_ds2 = [ds for ds in flow_ds1 if ds['dataset_id'] in stn_ds]

        # setattr(self, 'flow_datasets', flow_ds2)

        ## Remove existing attributes if they exist
        if hasattr(self, 'catch'):
            delattr(self, 'catch')
        if hasattr(self, 'waps'):
            delattr(self, 'waps')
        if hasattr(self, 'flow'):
            delattr(self, 'flow')
        if hasattr(self, 'usage_rate'):
            delattr(self, 'usage_rate')
        if hasattr(self, 'nat_flow'):
            delattr(self, 'nat_flow')

        return stns1


    # @staticmethod
    # def _get_catchment(inputs):
    #     """

    #     """
    #     station_id = inputs['station_id']
    #     bucket = inputs['bucket']
    #     public_url = inputs['public_url']

    #     key1 = catch_key_base.format(station_id=station_id)
    #     try:
    #         obj0 = utils.get_object_s3(key1, public_url=public_url, bucket=bucket, counter=1)
    #         dctx = zstd.ZstdDecompressor()
    #         obj1 = dctx.decompress(obj0)
    #         b2 = io.BytesIO(obj1)
    #         c1 = gpd.read_file(b2)
    #     except:
    #         c1 = gpd.GeoDataFrame(columns=['id', 'area', 'dataset_id', 'distance', 'nzsegment', 'ref', 'station_id', 'geometry'])

    #     return c1


    def get_catchments(self):
        """

        """
        if not hasattr(self, 'stations'):
            raise ValueError('Please run the process_stations method.')

        stns = self.stations.copy()

        w0 = nzrec.Water(self.nzrec_path)

        catch_list = []
        for i, row in stns.iterrows():
            geo = row.geometry
            stn_id = row.station_id
            coords = np.asarray(geo.coords[0])
            way0 = w0.nearest_way_within_catchments(coords, 0.1)
            up0 = way0.upstream()
            catch0 = up0.catchments().to_gpd()
            catch0['station_id'] = stn_id
            catch_list.append(catch0)

        catch1 = pd.concat(catch_list).reset_index(drop=True)

        setattr(self, 'catch', catch1)
        return catch1




        # stns = self.stations.copy()
        # stn_ids = stns.station_id.unique()

        # public_url = self.flow_remote['public_url']
        # bucket = self.flow_remote['bucket']

        # input_list = [{'public_url': public_url, 'bucket': bucket, 'station_id': s} for s in stn_ids]

        # output = ThreadPool(threads).map(self._get_catchment, input_list)

        # catch1 = pd.concat(output).drop('id', axis=1)
        # catch1.crs = pyproj.CRS(2193)
        # catch1 = catch1.to_crs(4326)

        # ## Save if required
        # if hasattr(self, 'output_path'):
        #     run_time = pd.Timestamp.today().strftime('%Y-%m-%dT%H%M')
        #     catch_del_shp = outputs['catch_del_shp'].format(run_date=run_time)
        #     catch1.to_file(os.path.join(self.output_path, catch_del_shp))

        # setattr(self, 'catch', catch1)
        # return catch1


    # def get_waps(self):
    #     """

    #     """
    #     tethys1 = Tethys([self.usage_remote])

    #     usage_ds = [ds for ds in tethys1.datasets if (ds['parameter'] == 'water_use') and (ds['product_code'] == 'raw_data') and (ds['frequency_interval'] == '24H') and (ds['utc_offset'] == '12H') and (ds['method'] == 'sensor_recording')]

    #     stns_list = []

    #     for ds in usage_ds:
    #         stns1 = tethys1.get_stations(ds['dataset_id'])
    #         stns_list.extend(stns1)

    #     stns_list3 = [{'dataset_id': s['dataset_id'], 'station_id': s['station_id'], 'ref': s['ref'], 'geometry': Point(s['geometry']['coordinates']), 'from_date': s['time_range']['from_date'], 'to_date': s['time_range']['to_date']} for s in stns_list]
    #     [s.update({'from_date': s['from_date'] + '+00:00', 'to_date': s['to_date'] + '+00:00'}) for s in stns_list3 if not '+00:00' in s['from_date']]

    #     stns_summ = gpd.GeoDataFrame(pd.DataFrame(stns_list3), geometry='geometry', crs=4326)

    #     try:
    #         stns_summ['from_date'] = pd.to_datetime(stns_summ['from_date']).dt.tz_convert(self.local_tz).dt.tz_localize(None)
    #         stns_summ['to_date'] = pd.to_datetime(stns_summ['to_date']).dt.tz_convert(self.local_tz).dt.tz_localize(None)
    #     except:
    #         pass

    #     if isinstance(self.from_date, str):
    #         from_date1 = pd.Timestamp(self.from_date)
    #         stns_summ = stns_summ[stns_summ['to_date'] >= from_date1]
    #     if isinstance(self.to_date, str):
    #         to_date1 = pd.Timestamp(self.to_date)
    #         stns_summ = stns_summ[stns_summ['from_date'] <= to_date1]

    #     setattr(self, 'waps_all', stns_summ)
    #     setattr(self, '_tethys_usage', tethys1)
    #     setattr(self, 'usage_datasets', usage_ds)

    #     return stns_summ


    def get_upstream_waps(self):
        """
        Function to determine the upstream water abstraction sites from the catchment delineation.

        Returns
        -------
        DataFrame
            allocation data
        """
        waps_all = self.waps_all.drop('permit_id', axis=1).drop_duplicates(subset='wap').to_crs(4326).copy()

        if not hasattr(self, 'catch'):
            catch1 = self.get_catchments()
        else:
            catch1 = self.catch.copy()

        # waps.rename(columns={'station_id': 'wap_stn_id', 'dataset_id': 'wap_ds_id', 'ref': 'wap'}, inplace=True)

        ### WAP selection
        waps_catch, poly1 = vector.pts_poly_join(waps_all, catch1, 'station_id')

        ### Get crc data
        if waps_catch.empty:
            raise ValueError('No WAPs were found in the polygon(s)')
        # else:
        #     waps_sel = waps_all[waps_all.wap.isin(waps_catch.wap.unique())].copy()

            ## Save if required
            # if hasattr(self, 'output_path'):
            #     run_time = pd.Timestamp.today().strftime('%Y-%m-%dT%H%M')

            #     save1 = waps_sel.copy()
            #     save1['from_date'] = save1['from_date'].astype(str)
            #     save1['to_date'] = save1['to_date'].astype(str)

            #     waps_shp = outputs['waps_shp'].format(run_date=run_time)
            #     save1.to_file(os.path.join(self.output_path, waps_shp))

        ## Return
        setattr(self, 'waps_catch', waps_catch)

        return waps_catch


    def get_usage(self, est_method='ratio', est_gw_sd_lags=False):
        """

        """
        if not hasattr(self, 'waps_catch'):
            waps_catch = self.get_upstream_waps()
        else:
            waps_catch = self.waps_catch.copy()

        # waps2 = waps_catch.groupby(['wap_ds_id', 'wap_stn_id']).first().reset_index()

        wap_ids = waps_catch.wap.unique().tolist()

        allo1 = AlloUsage(self.permits_path, self.usage_path, wap_filter={'wap': wap_ids}, from_date=self.from_date, to_date=self.to_date, use_type_mapping=self.use_type_mapping, default_sd_ratio=self.default_sd_ratio)

        usage1 = allo1.get_ts(['allo', 'sd_rates'], 'D', ['wap'], usage_est_method=est_method, est_gw_sd_lags=est_gw_sd_lags)
        usage1a = usage1[(usage1['total_allo'] > 0) & (usage1['sw_allo'] > 0)].dropna().copy()
        if 'sd_rate' not in usage1a.columns:
            usage1a['sd_rate'] = 0
        usage2 = usage1a[['sw_allo', 'sd_rate']].reset_index()

        usage3 = pd.merge(waps_catch[['wap', 'station_id']], usage2, on='wap')

        ## Aggregate by flow station id and date
        usage4 = usage3.groupby(['station_id', 'date'])[['sw_allo', 'sd_rate']].sum()
        usage5 = (usage4 / 24 / 60 / 60).round(3)

        usage5.rename(columns={'sw_allo': 'allocation', 'sd_rate': 'stream depletion'}, inplace=True)

        ## Aggregate by flow station id, wap station id, and date
        usage6 = usage3.groupby(['station_id', 'wap', 'date'])[['sw_allo', 'sd_rate']].sum()
        usage7 = (usage6 / 24 / 60 / 60).round(3)

        usage7.rename(columns={'sw_allo': 'allocation', 'sd_rate': 'stream depletion'}, inplace=True)

        ## Save results
        # if hasattr(self, 'output_path'):
        #     run_time = pd.Timestamp.today().strftime('%Y-%m-%dT%H%M')

        #     usage_rate_wap_csv = outputs['usage_rate_wap_csv'].format(run_date=run_time)
        #     usage1.to_csv(os.path.join(self.output_path, usage_rate_wap_csv))

        setattr(self, 'usage_rate', usage5.reset_index())
        setattr(self, 'usage_rate_wap', usage7.reset_index())

        return usage5.reset_index()


    def get_flow(self, buffer_dis=60000):
        """
        Function to query and/or estimate flow at the input_sites.

        Parameters
        ----------
        buffer_dis : int
            The search radius for the regressions in meters.

        Returns
        -------
        DataFrame of Flow
        """
        flow_list = []
        with booklet.open(self.flow_data_path) as f:
            # print(list(f.keys()))
            for stn_id, data in f.items():
                data0 = data.set_index('time').loc[slice(self.from_date, self.to_date)]
                if not data0.empty:
                    flow_list.append(data0)

        flow = pd.concat(flow_list, axis=1)

        setattr(self, 'flow', flow)
        # setattr(self, 'reg_flow', reg_df)
        return flow




        # ### Prep the stations and other inputs
        # flow_ds = self.flow_datasets.copy()
        # tethys1 = self._tethys_flow
        # stns = self.stations.copy()
        # rec_ds_id = [ds for ds in self.flow_datasets_all if ds['method'] == 'sensor_recording'][0]['dataset_id']
        # man_ds_id = [ds for ds in self.flow_datasets_all if ds['method'] == 'field_activity'][0]['dataset_id']

        # methods = [m['method'] for m in flow_ds]

        # rec_stns = self.stations_all[self.stations_all.dataset_id == rec_ds_id].to_crs(2193).copy()

        # if self.from_date is None:
        #     from_date1 = None
        # else:
        #     from_date1 = pd.Timestamp(self.from_date, tz=self.local_tz).tz_convert('utc').tz_localize(None)
        # if self.to_date is None:
        #     to_date1 = None
        # else:
        #     to_date1 = pd.Timestamp(self.to_date, tz=self.local_tz).tz_convert('utc').tz_localize(None)

        # ### Iterate through the two datasets
        # for ds in flow_ds:
        #     ds_id = ds['dataset_id']
        #     stns1 = stns[stns['dataset_id'] == ds_id].copy()
        #     stn_ids = stns1['station_id'].unique().tolist()

        #     flow_data1 = tethys1.get_results(ds_id, stn_ids, from_date=from_date1, to_date=to_date1, squeeze_dims=True, threads=threads)

        #     val2 = flow_data1[['streamflow', 'station_id']].drop('height').to_dataframe().reset_index()
        #     flow_data = val2.drop('geometry', axis=1).dropna()

        #     flow_data['time'] = flow_data['time'].dt.tz_localize('utc').dt.tz_convert(self.local_tz).dt.tz_localize(None).dt.floor('D')

        #     flow_data = flow_data.groupby(['station_id', 'time']).mean().reset_index()

        #     if ds['method'] == 'field_activity':

        #         man_ts_data2 = flow_data.set_index(['station_id', 'time'])['streamflow'].unstack(0)

        #         ## Determine which sites are within the buffer of the manual sites
        #         man_stns = stns[stns['dataset_id'] == man_ds_id].to_crs(2193).copy()
        #         man_stns.set_index('station_id', inplace=True)
        #         man_stns['geometry'] = man_stns.buffer(buffer_dis)

        #         buff_sites_dict = {}

        #         for index in man_stns.index:
        #             buff_sites1 = vector.sel_sites_poly(rec_stns, man_stns.loc[[index]])
        #             buff_sites_dict[index] = buff_sites1['station_id'].tolist()

        #         buff_sites_list = [item for sublist in buff_sites_dict.values() for item in sublist]
        #         buff_sites = list(set(buff_sites_list))

        #         ## Pull out recorder data needed for all manual sites
        #         rec_data1 = tethys1.get_results(rec_ds_id, buff_sites, from_date=from_date1, to_date=to_date1, squeeze_dims=True, threads=threads)

        #         val2 = rec_data1[['streamflow', 'station_id']].drop('height').to_dataframe().reset_index()
        #         rec_data = val2.drop('geometry', axis=1).dropna()

        #         rec_data['time'] = rec_data['time'].dt.tz_localize('utc').dt.tz_convert(self.local_tz).dt.tz_localize(None).dt.floor('D')
        #         rec_data1 = rec_data.groupby(['station_id', 'time']).mean()
        #         rec_data1 = rec_data1['streamflow'].unstack(0).interpolate('time', limit=10)

        #         ## Run through regressions
        #         reg_lst = []
        #         new_lst = []

        #         for key, lst in buff_sites_dict.items():
        #             # print(key)
        #             man_rec_ts_data3 = rec_data1.loc[:, rec_data1.columns.isin(lst)].copy()
        #             man_rec_ts_data3[man_rec_ts_data3 <= 0] = np.nan

        #             man_ts_data3 = man_ts_data2.loc[:, [key]].copy()
        #             man_ts_data3[man_ts_data3 <= 0] = np.nan

        #             lm1 = LM(man_rec_ts_data3, man_ts_data3)
        #             res1 = lm1.predict(n_ind=1, x_transform='log', y_transform='log', min_obs=self.min_gaugings)
        #             if res1 is None:
        #                 continue

        #             res1_f = res1.summary_df['f value'].iloc[0]
        #             res2 = lm1.predict(n_ind=2, x_transform='log', y_transform='log', min_obs=self.min_gaugings)
        #             if res2 is not None:
        #                 res2_f = res2.summary_df['f value'].iloc[0]
        #             else:
        #                 res2_f = 0

        #             f = [res1_f, res2_f]

        #             val = f.index(max(f))

        #             if val == 0:
        #                 reg_lst.append(res1.summary_df)

        #                 s1 = res1.summary_df.iloc[0]

        #                 d1 = man_rec_ts_data3[s1['x sites']].copy()
        #                 d1[d1 <= 0] = 0.001

        #                 new_data1 = np.exp(np.log(d1) * float(s1['x slopes']) + float(s1['y intercept']))
        #                 new_data1.name = key
        #                 new_data1[new_data1 <= 0] = 0
        #             else:
        #                 reg_lst.append(res2.summary_df)

        #                 s1 = res2.summary_df.iloc[0]
        #                 x_sites = s1['x sites'].split(', ')
        #                 x_slopes = [float(s) for s in s1['x slopes'].split(', ')]
        #                 intercept = float(s1['y intercept'])

        #                 d1 = man_rec_ts_data3[x_sites[0]].copy()
        #                 d1[d1 <= 0] = 0.001
        #                 d2 = man_rec_ts_data3[x_sites[1]].copy()
        #                 d2[d2 <= 0] = 0.001

        #                 new_data1 = np.exp((np.log(d1) * float(x_slopes[0])) + (np.log(d2) * float(x_slopes[1])) + intercept)
        #                 new_data1.name = key
        #                 new_data1[new_data1 <= 0] = 0

        #             new_lst.append(new_data1)

        #         new_data2 = pd.concat(new_lst, axis=1)
        #         reg_df = pd.concat(reg_lst).reset_index()

        #     elif ds['method'] == 'sensor_recording':
        #         rec_flow_data = flow_data.set_index(['station_id', 'time'])['streamflow'].unstack(0)
        #     else:
        #         raise ValueError('The dataset method should be either field_activity or sensor_recording.')

        # if not 'sensor_recording' in methods:
        #     flow = new_data2.round(3)
        # elif not 'field_activity' in methods:
        #     flow = rec_flow_data.round(3)
        #     reg_df = pd.DataFrame()
        # else:
        #     flow = pd.concat([rec_flow_data, new_data2], axis=1).round(3)

        # ## Save if required
        # if hasattr(self, 'output_path'):
        #     run_time = pd.Timestamp.today().strftime('%Y-%m-%dT%H%M')

        #     # stns

        #     if not reg_df.empty:
        #         reg_flow_csv = outputs['reg_flow_csv'].format(run_date=run_time)
        #         reg_df.to_csv(os.path.join(self.output_path, reg_flow_csv), index=False)

        #     flow_csv = outputs['flow_csv'].format(run_date=run_time)
        #     flow.to_csv(os.path.join(self.output_path, flow_csv))

        # setattr(self, 'flow', flow)
        # setattr(self, 'reg_flow', reg_df)
        # return flow


    def naturalisation(self, est_method='ratio', est_gw_sd_lags=False):
        """
        Function to put all of the previous functions together to estimate the naturalised flow at the input_sites. It takes the estimated usage rates above each input site and adds that back to the flow.

        Returns
        -------
        DataFrame
            of measured flow, upstream usage rate, and naturalised flow
        """
        if not hasattr(self, 'usage_rate'):
            usage_daily_rate = self.get_usage(est_method=est_method, est_gw_sd_lags=est_gw_sd_lags)
        else:
            usage_daily_rate = self.usage_rate.copy()

        if not hasattr(self, 'flow'):
            flow = self.get_flow()
        else:
            flow = self.flow.copy()

        if usage_daily_rate.empty:
            flow2 = flow.stack().reset_index()
            flow2.columns = ['date', 'station_id', 'flow']
            flow2 = flow2.set_index(['station_id', 'date']).sort_index()
            flow2['sw_usage'] = 0
        else:

            ## Add usage to flow
            flow1 = flow.stack().reset_index()
            flow1.columns = ['date', 'station_id', 'flow']

            flow2 = pd.merge(flow1, usage_daily_rate, on=['station_id', 'date'], how='left').set_index(['station_id', 'date']).sort_index()
            flow2.loc[flow2['stream depletion'].isnull(), 'stream depletion'] = 0
            # flow2.loc[flow2['estimated usage'].isnull(), 'estimated usage'] = 0
            flow2.loc[flow2['allocation'].isnull(), 'allocation'] = 0

        flow2['nat flow'] = flow2['flow'] + flow2['stream depletion']

        ## Use the reference identifier instead of station_ids
        flow3 = pd.merge(flow2.reset_index(), self.stations[['station_id', 'ref']], on='station_id').drop('station_id', axis=1)
        flow4 = flow3.groupby(['ref', 'date']).mean().sort_index()

        # nat_flow = flow4.unstack(0).round(3)

        ## Save results
        # if hasattr(self, 'output_path'):
        #     run_time = pd.Timestamp.today().strftime('%Y-%m-%dT%H%M')

        #     nat_flow_csv = outputs['nat_flow_csv'].format(run_date=run_time)
        #     nat_flow.to_csv(os.path.join(self.output_path, nat_flow_csv))

        #     setattr(self, 'nat_flow_csv', nat_flow_csv)

        setattr(self, 'nat_flow', flow4)
        return flow4


    # def plot(self, input_site):
    #     """
    #     Function to run and plot the detide results.

    #     Parameters
    #     ----------
    #     output_path : str
    #         Path to save the html file.

    #     Returns
    #     -------
    #     DataFrame or Series
    #     """

    #     if hasattr(self, 'nat_flow'):
    #         nat_flow = self.nat_flow.copy()
    #     else:
    #         nat_flow = self.naturalisation()

    #     nat_flow1 = nat_flow.loc[:, (slice(None), input_site)]
    #     nat_flow1.columns = nat_flow1.columns.droplevel(1)

    #     colors1 = ['rgb(102,194,165)', 'rgb(252,141,98)', 'rgb(252,141,0)', 'rgb(141,160,203)']

    #     orig = go.Scattergl(
    #         x=nat_flow1.index,
    #         y=nat_flow1['flow'],
    #         name = 'Recorded Flow',
    #         line = dict(color = colors1[3]),
    #         opacity = 0.8)

    #     meas_usage = go.Scattergl(
    #         x=nat_flow1.index,
    #         y=nat_flow1['stream depletion'],
    #         name = 'Stream Depletion',
    #         line = dict(color = colors1[1]),
    #         opacity = 0.8)

    #     # est_usage = go.Scattergl(
    #     #     x=nat_flow1.index,
    #     #     y=nat_flow1['estimated usage'],
    #     #     name = 'Estimated Stream Usage',
    #     #     line = dict(color = colors1[2]),
    #     #     opacity = 0.8)

    #     est_usage = go.Scattergl(
    #         x=nat_flow1.index,
    #         y=nat_flow1['allocation'],
    #         name = 'Allocation',
    #         line = dict(color = colors1[2]),
    #         opacity = 0.8)

    #     nat = go.Scattergl(
    #         x=nat_flow1.index,
    #         y=nat_flow1['nat flow'],
    #         name = 'Naturalised Flow',
    #         line = dict(color = colors1[0]),
    #         opacity = 0.8)

    #     data = [orig, meas_usage, est_usage, nat]

    #     layout = dict(
    #         title=input_site + ' Naturalisation',
    #         yaxis={'title': 'Flow rate (m3/s)'},
    #         dragmode='pan')

    #     config = {"displaylogo": False, 'scrollZoom': True, 'showLink': False}

    #     fig = dict(data=data, layout=layout)

    #     ## Save results
    #     # if hasattr(self, 'output_path'):
    #     #     run_time = pd.Timestamp.today().strftime('%Y-%m-%dT%H%M')

    #     #     nat_flow_html = outputs['nat_flow_html'].format(site=input_site, run_date=run_time)
    #     #     py.plot(fig, filename = os.path.join(self.output_path, nat_flow_html), config=config)
    #     # else:
    #     #     raise ValueError('plot must have an output_path set')

    #     return nat_flow1



# -*- coding: utf-8 -*-
"""
Created on Fri Apr 26 14:47:14 2019

@author: michaelek
"""
import os
import pandas as pd
from allotools import AlloUsage
from pdsql import mssql
import upstream_takes as takes
from parameters import param

pd.options.display.max_columns = 10

###################################
### Parameters

grp = ['Wap', 'WaterUse']
datasets = ['Allo', 'RestrAllo', 'Usage']

use_type_dict = {'industrial': 'other', 'municipal': 'other'}

server = param['ts_server']
database = 'hydro'
sites_table = 'ExternalSite'
results_path = param['results_path']

swaz_mon_ratio_csv = 'swaz_mon_ratio_{}.csv'.format(param['run_time'])
allo_usage_wap_swaz_csv = 'allo_usage_wap_swaz_{}.csv'.format(param['run_time'])
wap_sw_mon_usage_csv = 'wap_sw_monthly_usage_rate_{}.csv'.format(param['run_time'])
wap_sw_daily_usage_hdf = 'wap_sw_daily_usage_rate_{}.hd5'.format(param['run_time'])

###################################
### Read existing usage data
print('Read in usage estimates')

try:
    res_swaz5 = pd.read_csv(os.path.join(results_path, swaz_mon_ratio_csv))
    usage4 = pd.read_csv(os.path.join(results_path, allo_usage_wap_swaz_csv), parse_dates=['Date'], infer_datetime_format=True)
    usage_rate = pd.read_csv(os.path.join(results_path, wap_sw_mon_usage_csv), parse_dates=['Date'], infer_datetime_format=True)

    print('-> loaded from local files')

except:

    print('-> Processing usage data from the databases')

    allo1 = AlloUsage(param['from_date'], param['to_date'], site_filter={'SwazGroupName': takes.waps_gdf.SwazGroupName.unique().tolist()})

    usage1 = allo1.get_ts(datasets, 'M', grp)

    usage2 = usage1.loc[usage1.SwRestrAllo > 0, ['SwRestrAllo', 'SwUsage']].reset_index().copy()

    usage2.replace({'WaterUse': use_type_dict}, inplace=True)

    usage2[['SwRestrAlloYr', 'SwUsageYr']] = usage2.groupby(['Wap', 'WaterUse', pd.Grouper(key='Date', freq='A-JUN')]).transform('sum')

    sites1 = mssql.rd_sql(server, database, sites_table, ['ExtSiteID', 'SwazGroupName', 'SwazName'], where_in={'ExtSiteID': usage2.Wap.unique().tolist()})
    sites1.rename(columns={'ExtSiteID': 'Wap'}, inplace=True)

    usage0 = pd.merge(sites1, usage2, on='Wap')
    usage0['Mon'] = usage0.Date.dt.month

    usage0['MonRatio'] = usage0.SwUsage/usage0.SwRestrAllo
    usage0['YrRatio'] = usage0.SwUsageYr/usage0.SwRestrAlloYr

    usage0.set_index(['Wap', 'Date', 'WaterUse'], inplace=True)

    filter1 = (usage0['YrRatio'] >= 0.04) & (usage0['YrRatio'] <= 2) & (usage0['MonRatio'] >= 0.001)

    usage3 = usage0[filter1].reset_index().copy()

    res_swaz1 = usage3.groupby(['SwazGroupName', 'SwazName', 'WaterUse', 'Mon']).MonRatio.mean()
    res_grp1 = usage3.groupby(['SwazGroupName', 'WaterUse', 'Mon']).MonRatio.mean()
    res_grp1.name = 'GrpRatio'

    res_grp2 = usage3.groupby(['WaterUse', 'Mon']).MonRatio.mean()
    res_grp2.name = 'GrossRatio'

    all1 = usage0.groupby(['SwazGroupName', 'SwazName', 'WaterUse', 'Mon']).Mon.first()

    res_swaz2 = pd.concat([res_swaz1, all1], axis=1).drop('Mon', axis=1)
    res_swaz3 = pd.merge(res_swaz2.reset_index(), res_grp1.reset_index(), on=['SwazGroupName', 'WaterUse', 'Mon'], how='left')
    res_swaz4 = pd.merge(res_swaz3, res_grp2.reset_index(), on=['WaterUse', 'Mon'], how='left')

    res_swaz4.loc[res_swaz4.MonRatio.isnull(), 'MonRatio'] = res_swaz4.loc[res_swaz4.MonRatio.isnull(), 'GrpRatio']

    res_swaz4.loc[res_swaz4.MonRatio.isnull(), 'MonRatio'] = res_swaz4.loc[res_swaz4.MonRatio.isnull(), 'GrossRatio']

    res_swaz5 = res_swaz4.drop(['GrpRatio', 'GrossRatio'], axis=1).copy()

    ### Estimate monthly usage by WAP

    usage4 = pd.merge(usage0.drop(['MonRatio', 'YrRatio', 'SwRestrAlloYr', 'SwUsageYr'], axis=1).reset_index(), res_swaz5, on=['SwazGroupName', 'SwazName', 'WaterUse', 'Mon'], how='left').set_index(['Wap', 'Date', 'WaterUse'])

    usage4.loc[~filter1, 'SwUsage'] = usage4.loc[~filter1, 'SwRestrAllo'] * usage4.loc[~filter1, 'MonRatio']

    usage_rate = usage4.groupby(level=['Wap', 'Date'])[['SwUsage']].sum().reset_index().copy()
    usage_rate.rename(columns={'SwUsage': 'SwUsageRate'}, inplace=True)

    days1 = usage_rate.Date.dt.daysinmonth
    usage_rate['SwUsageRate'] = usage_rate['SwUsageRate'] / days1 /24/60/60

    usage4.reset_index(inplace=True)

    ### Save results

    res_swaz5.to_csv(os.path.join(results_path, swaz_mon_ratio_csv), index=False)
    usage4.to_csv(os.path.join(results_path, allo_usage_wap_swaz_csv), index=False)
    usage_rate.to_csv(os.path.join(results_path, wap_sw_mon_usage_csv), index=False)





# -*- coding: utf-8 -*-
"""
Created on Mon Apr 29 12:20:30 2019

@author: michaelek
"""
import os
import pandas as pd
import upstream_takes as takes
import usage_estimates as ue
import flow_estimates as flow
from parameters import param

pd.options.display.max_columns = 10

#######################################
### Parameters

flow_site_sw_usage_csv = 'flow_site_sw_usage_rate_{}.csv'.format(param['run_time'])
nat_flow_csv = 'nat_flow_{}.csv'.format(param['run_time'])

#####################################
### Naturalisation

print('Naturalise the flow data')

### Resample to daily rate
waps1 = takes.waps_gdf.drop(['geometry', 'SwazGroupName', 'SwazName'], axis=1).copy()

usage_rate = ue.usage_rate[ue.usage_rate.Wap.isin(waps1.Wap.unique())].copy()

days1 = usage_rate.Date.dt.daysinmonth
days2 = pd.to_timedelta((days1/2).round().astype('int32'), unit='D')

usage_rate0 = usage_rate.copy()

usage_rate0['Date'] = usage_rate0['Date'] - days2

grp1 = usage_rate.groupby('Wap')
first1 = grp1.first()
last1 = grp1.last()

first1.loc[:, 'Date'] = pd.to_datetime(first1.loc[:, 'Date'].dt.strftime('%Y-%m') + '-01')

usage_rate1 = pd.concat([first1, usage_rate0.set_index('Wap'), last1], sort=True).reset_index()

usage_rate1.set_index('Date', inplace=True)

usage_daily_rate = usage_rate1.groupby('Wap').apply(lambda x: x.resample('D').interpolate(method='pchip')['SwUsageRate']).reset_index()

## Combine usage with site data

print('-> Combine usage with site data')

usage_rate3 = pd.merge(waps1, usage_daily_rate.reset_index(), on='Wap')

site_rate = usage_rate3.groupby(['FlowSite', 'Date'])[['SwUsageRate']].sum().reset_index()

site_rate.to_csv(os.path.join(param['results_path'], flow_site_sw_usage_csv), index=False)

## Add usage to flow
print('-> Add usage to flow')

flow1 = flow.flow.stack().reset_index()
flow1.columns = ['Date', 'FlowSite', 'Flow']

flow2 = pd.merge(flow1, site_rate, on=['FlowSite', 'Date'], how='left').set_index(['FlowSite', 'Date']).sort_index()
flow2.loc[flow2.SwUsageRate.isnull(), 'SwUsageRate'] = 0

flow2['NatFlow'] = flow2['Flow'] + flow2['SwUsageRate']

nat_flow = flow2.unstack(0).round(3)

nat_flow.to_csv(os.path.join(param['results_path'], nat_flow_csv))



############################################
### Testing





# -*- coding: utf-8 -*-
"""
Created on Wed Apr 24 09:32:35 2019

@author: michaelek
"""
import os
import pandas as pd
import geopandas as gpd
from gistools import rec, vector
from ecandbparams import sql_arg
from allotools import AlloUsage
from pdsql import mssql
from parameters import param

pd.options.display.max_columns = 10

#######################################
### Parameters

site_csv = 'sites.csv'

ts_server = param['ts_server']
permit_server = param['permit_server']
ts_database = 'hydro'
permit_database = 'ConsentsReporting'
sites_table = 'ExternalSite'
crc_wap_table = 'reporting.CrcAlloSiteSumm'
results_path = param['results_path']

rec_rivers_sql = 'rec_rivers_gis'
rec_catch_sql = 'rec_catch_gis'

catch_del_shp = 'catch_del_{}.shp'.format(param['run_time'])
catch_del_base = 'catch_del_'
allo_csv = 'allo_{}.csv'.format(param['run_time'])
waps_shp = 'waps_{}.shp'.format(param['run_time'])
flow_sites_shp = 'flow_sites_{}.shp'.format(param['run_time'])

######################################
### Read in data
print('Read in allocation and sites data')

try:
    flow_sites_gdf = gpd.read_file(os.path.join(results_path, flow_sites_shp))
    waps_gdf = gpd.read_file(os.path.join(results_path, waps_shp))
    waps_gdf.rename(columns={'SwazGroupN': 'SwazGroupName', 'min_flow_s': 'min_flow_site'}, inplace=True)
    allo = pd.read_csv(os.path.join(results_path, allo_csv))

    print('-> loaded from local files')

except:
    print('-> Processing data from the databases')

    sites1 = mssql.rd_sql(ts_server, ts_database, sites_table, ['ExtSiteID', 'NZTMX', 'NZTMY', 'SwazGroupName', 'SwazName'])

    input_sites1 = pd.read_csv(os.path.join(param['inputs_path'], site_csv)).site.astype(str)

    sites0 = sites1[sites1.ExtSiteID.isin(input_sites1)].copy()
    sites0.rename(columns={'ExtSiteID': 'FlowSite'}, inplace=True)

    flow_sites_gdf = vector.xy_to_gpd('FlowSite', 'NZTMX', 'NZTMY', sites0)
    flow_sites_gdf.to_file(os.path.join(results_path, flow_sites_shp))

    ###################################
    ### Catchment delineation

    try:
        catch_del_shp = [p for p in os.listdir(results_path) if (catch_del_base in p) and ('.shp' in p)][-1]
        catch_gdf = gpd.read_file(os.path.join(results_path, catch_del_shp))
    except:
        sql1 = sql_arg()

        rec_rivers_dict = sql1.get_dict(rec_rivers_sql)
        rec_catch_dict = sql1.get_dict(rec_catch_sql)

        rec_rivers = mssql.rd_sql(**rec_rivers_dict)
        rec_catch = mssql.rd_sql(**rec_catch_dict)

        catch_gdf = rec.catch_delineate(flow_sites_gdf, rec_rivers, rec_catch)
        catch_gdf.to_file(os.path.join(results_path, catch_del_shp))

    ###################################
    ### WAP selection

    wap1 = mssql.rd_sql(permit_server, permit_database, crc_wap_table, ['ExtSiteID'], where_in={'ConsentStatus': param['crc_status']}).ExtSiteID.unique()

    sites3 = sites1[sites1.ExtSiteID.isin(wap1)].copy()
    sites3.rename(columns={'ExtSiteID': 'Wap'}, inplace=True)

    sites4 = vector.xy_to_gpd('Wap', 'NZTMX', 'NZTMY', sites3)
    sites4 = sites4.merge(sites3.drop(['NZTMX', 'NZTMY'], axis=1), on='Wap')

    waps_gdf, poly1 = vector.pts_poly_join(sites4, catch_gdf, 'FlowSite')
    waps_gdf.to_file(os.path.join(results_path, waps_shp))

    ##################################
    ### Get crc data

    allo1 = AlloUsage(crc_filter={'ExtSiteID': waps_gdf.Wap.unique().tolist(), 'ConsentStatus': param['crc_status']}, from_date=param['from_date'], to_date=param['to_date'])

    allo_wap1 = allo1.allo.copy()
    allo_wap = pd.merge(allo_wap1.reset_index(), waps_gdf[['Wap', 'FlowSite']], on='Wap')

    allo_wap.to_csv(os.path.join(results_path, allo_csv), index=False)




# -*- coding: utf-8 -*-
"""
Created on Mon Apr 29 15:44:32 2019

@author: michaelek
"""
import os
import numpy as np
import pandas as pd
from pdsql import mssql
import geopandas as gpd
import upstream_takes as takes
from hydrolm import LM
from gistools import vector
from parameters import param

pd.options.display.max_columns = 10

#######################################
### Parameters

server = param['ts_server']
database = 'hydro'
site_table = 'ExternalSite'
ts_table = 'TSDataNumericDaily'
ts_summ_table = 'TSDataNumericDailySumm'
dataset_type_table = 'DatasetType'
results_path = param['results_path']

buffer_dis = param['buffer_dis']

flow_csv = 'flow_data_{}.csv'.format(param['run_time'])
reg_flow_csv = 'reg_flow_{}.csv'.format(param['run_time'])


#####################################
### Estimate flow data

print('Estimate flow data')

try:

    flow = pd.read_csv(os.path.join(param.results_path, flow_csv), parse_dates=['DateTime'], infer_datetime_format=True, index_col='DateTime')
    reg_df = pd.read_csv(os.path.join(param.results_path, reg_flow_csv))

    print('-> loaded from local files')

except:

    print('-> processing with regressions')

    ## Read in data
    datasets = mssql.rd_sql(server, database, dataset_type_table, ['DatasetTypeID', 'CTypeID'], where_in={'FeatureID': [1], 'MTypeID': [2], 'CTypeID': [1, 2], 'DataCodeID': [1]})

    site_summ1 = mssql.rd_sql(server, database, ts_summ_table, where_in={'DatasetTypeID': datasets.DatasetTypeID.tolist()})
    site_summ1.FromDate = pd.to_datetime(site_summ1.FromDate)
    site_summ1.ToDate = pd.to_datetime(site_summ1.ToDate)

    rec_datasets = datasets[datasets.CTypeID == 1].DatasetTypeID.tolist()
    man_datasets = datasets[datasets.CTypeID == 2].DatasetTypeID.tolist()

    rec_summ1 = site_summ1[site_summ1.DatasetTypeID.isin(rec_datasets) & (site_summ1.FromDate <= param['from_date']) & (site_summ1.ToDate >= param['to_date'])].sort_values('ToDate', ascending=False).drop_duplicates('ExtSiteID').copy()

    flow_sites_gdf = takes.flow_sites_gdf.copy()

    sites_rec_bool = flow_sites_gdf.FlowSite.isin(rec_summ1.ExtSiteID.unique())

    sites_rec1 = flow_sites_gdf[sites_rec_bool].copy()
    sites_man1 = flow_sites_gdf[~sites_rec_bool].copy()

    flow_rec_sites1 = mssql.rd_sql(server, database, site_table, ['ExtSiteID', 'NZTMX', 'NZTMY'], where_in={'ExtSiteID': rec_summ1.ExtSiteID.unique().tolist()})

    flow_rec_sites2 = vector.xy_to_gpd('ExtSiteID', 'NZTMX', 'NZTMY', flow_rec_sites1)

    ## Estimate flow where recorder doesn't exist

    sites_man2 = sites_man1.copy()
    sites_man2['geometry'] = sites_man1.buffer(buffer_dis)

    rec_sites2 = vector.sel_sites_poly(flow_rec_sites2, sites_man2)

    rec_ts_data1 = mssql.rd_sql_ts(server, database, ts_table, 'ExtSiteID', 'DateTime', 'Value', from_date=param['from_date'], to_date=param['to_date'], where_in={'ExtSiteID': rec_sites2.ExtSiteID.tolist(), 'DatasetTypeID': rec_summ1.DatasetTypeID.unique().tolist()})

    rec_ts_data2 = rec_ts_data1.Value.unstack(0).interpolate('time', limit=10).dropna(axis=1)

    rec_flow1 = rec_ts_data2.loc[:, rec_ts_data2.columns.isin(sites_rec1.FlowSite)].copy()

    man_ts_data1 = mssql.rd_sql_ts(server, database, ts_table, 'ExtSiteID', 'DateTime', 'Value', from_date=param['from_date'], to_date=param['to_date'], where_in={'ExtSiteID': sites_man1.FlowSite.tolist(), 'DatasetTypeID': man_datasets})

    man_ts_data2 = man_ts_data1.Value.unstack(0)

    reg_lst = []
    new_lst = []

    for col in man_ts_data2:
        site0 = sites_man1[sites_man1.FlowSite == col]

        site1 = gpd.GeoDataFrame(geometry=site0.buffer(buffer_dis))

        rec_sites3 = vector.sel_sites_poly(flow_rec_sites2, site1)
        rec_ts_data3 = rec_ts_data2.loc[:, rec_ts_data2.columns.isin(rec_sites3.ExtSiteID)].copy()

        rec_ts_data4 = rec_ts_data3.copy()
        rec_ts_data4[rec_ts_data4 <= 0] = np.nan

        man_ts_data3 = man_ts_data2.loc[:, [site0.FlowSite.iloc[0]]].copy()
        man_ts_data3[man_ts_data3 <= 0] = np.nan

        lm1 = LM(rec_ts_data4, man_ts_data3)
        res1 = lm1.predict(n_ind=1, x_transform='log', y_transform='log', min_obs=param['min_gaugings'])
        res2 = lm1.predict(n_ind=2, x_transform='log', y_transform='log', min_obs=param['min_gaugings'])

        f = [res1.summary_df['f value'].iloc[0], res2.summary_df['f value'].iloc[0]]

        val = f.index(max(f))

        if val == 0:
            reg_lst.append(res1.summary_df)

            s1 = res1.summary_df.iloc[0]

            d1 = rec_ts_data3[s1['x sites']].copy()
            d1[d1 <= 0] = 0.001

            new_data1 = np.exp(np.log(d1) * float(s1['x slopes']) + float(s1['y intercept']))
            new_data1.name = col
            new_data1[new_data1 <= 0] = 0
        else:
            reg_lst.append(res2.summary_df)

            s1 = res2.summary_df.iloc[0]
            x_sites = s1['x sites'].split(', ')
            x_slopes = [float(s) for s in s1['x slopes'].split(', ')]
            intercept = float(s1['y intercept'])

            d1 = rec_ts_data3[x_sites[0]].copy()
            d1[d1 <= 0] = 0.001
            d2 = rec_ts_data3[x_sites[1]].copy()
            d2[d2 <= 0] = 0.001

            new_data1 = np.exp((np.log(d1) * float(x_slopes[0])) + (np.log(d2) * float(x_slopes[1])) + intercept)
            new_data1.name = col
            new_data1[new_data1 <= 0] = 0

        new_lst.append(new_data1)

    new_data2 = pd.concat(new_lst, axis=1)
    reg_df = pd.concat(reg_lst).reset_index()

    flow = pd.concat([rec_flow1, new_data2], axis=1)

    flow.round(3).to_csv(os.path.join(results_path, flow_csv))
    reg_df.to_csv(os.path.join(results_path, reg_flow_csv), index=False)





# -*- coding: utf-8 -*-
"""
Created on Wed Jul 17 17:08:09 2019

@author: michaelek
"""
import os
import numpy as np
from pdsql import mssql
from gistools import rec, vector
import pandas as pd
from flownat import FlowNat
import geopandas as gpd

pd.options.display.max_columns = 10

####################################
### Parameters

min_gaugings = 8
rec_data_code = 'Primary'
output_path = r'C:\ecan\shared\projects\catchment_delineation'
shp1 = 'catchment_delineation_rec_2019-07-17b.shp'
gpkg1 = 'catchment_delineation_rec_2019-07-17b.gpkg'
geojson1 = 'catchment_delineation_rec_2019-07-17b.geojson'
pkl1 = 'catchment_delineation_rec_2019-07-17b.pkl'
pkl2 = 'catchment_delineation_rec_2019-07-17b.pkl.xz'
pkl3 = 'catchment_delineation_rec_2019-07-17b.pkl.bz2'
pkl4 = 'catchment_delineation_rec_2019-07-17b.pkl.gzip'
pkl5 = 'catchment_delineation_rec_2019-07-17b.pkl.zip'

pkl2c = 'catchment_delineation_rec_2019-07-17c.pkl.xz'
pkl2f = 'rec_2-4b.pkl.xz'
pkl2g = 'rec_catch_2-4b.pkl.xz'

rec_rivers_path = r'P:\WaterDataProgramme\Source Data\GDBs\river-environment-classification-canterbury-2010\river-environment-classification-canterbury-2010.gpkg'
rec_catch_path = r'P:\WaterDataProgramme\Source Data\GDBs\river-environment-classification-watershed-canterbury-2010\river-environment-classification-watershed-canterbury-2010.gpkg'

###################################
###

f1 = FlowNat(output_path=output_path, load_rec=True)

sites1 = f1.summ[['ExtSiteID', 'NZTMX', 'NZTMY']].copy()

sites_gdf = vector.xy_to_gpd('ExtSiteID', 'NZTMX', 'NZTMY', sites1)

## Catch del
catch_gdf = catch_delineate(sites_gdf, f1.rec_rivers, f1.rec_catch)

catch_gdf.to_file(os.path.join(output_path, shp1))

c2 = catch_gdf.drop(['NZREACH', 'area'], axis=1).copy()
c2['geometry'] = catch_gdf.simplify(30)

c2.plot()

catch_gdf.plot()
c2.to_file(os.path.join(output_path, gpkg1), driver='GPKG')
#
### Save if required
#if hasattr(self, 'output_path'):
#    run_time = pd.Timestamp.today().strftime('%Y-%m-%dT%H%M')
#    catch_del_shp = param['output']['catch_del_shp'].format(run_date=run_time)
#    catch_gdf.to_file(os.path.join(self.output_path, catch_del_shp))
#
### Return
#setattr(self, 'catch_gdf', catch_gdf)
#return catch_gdf


c3 = vector.multipoly_to_poly(c2)
c3.to_file(os.path.join(output_path, gpkg1), driver='GPKG')

c3.to_file(os.path.join(output_path, geojson1), driver='GeoJSON')

c3.to_pickle(os.path.join(output_path, pkl1))

df1 = pd.read_pickle(os.path.join(output_path, pkl1))

c3.to_pickle(os.path.join(output_path, pkl2))


for p in [pkl1, pkl2, pkl3, pkl4, pkl5]:
#    %timeit c3.to_pickle(os.path.join(output_path, p))
    df1 = pd.read_pickle(os.path.join(output_path, p))

df1 = pd.read_pickle(os.path.join(output_path, pkl2))


c2.to_pickle(os.path.join(output_path, pkl2))


rec_rivers.to_pickle(os.path.join(output_path, pkl2f))
rec_catch.to_pickle(os.path.join(output_path, pkl2g))

rec_catch1 = rec_catch[rec_catch.NZREACH.isin(rec_rivers.NZREACH)]


rec_r1 = gpd.read_file(rec_rivers_path)
rec_r2 = rec_r1[['NZREACH', 'NZFNODE', 'NZTNODE', 'ORDER', 'geometry']]

rec_c1 = gpd.read_file(rec_catch_path)
rec_c2 = rec_c1[['NZREACH', 'geometry']]

rec_r2.to_pickle(os.path.join(output_path, pkl2f))
rec_c2.to_pickle(os.path.join(output_path, pkl2g))




